import java.util.ArrayList;

public class Roi extends Piece {
    public Roi(String couleur){
        super(couleur);
    }

    @Override
    public boolean isRoi() {
        return true;
    }


    @Override
    public ArrayList<Case> getCasesPossibles(Echiquier ech) {

        /*
         * Par rapport aux autres pièces, ici il faudrait vérifier après avoir récupéré
         * lesdéplacements possibles qu'il n'y a pas de mise en échec (en utilisant
         * isEchec de Echiquier).
         */

        // on calcule les cases atteignables sans tenir compte des échecs
        ArrayList<Case> array = getDepPossibles(ech);

        return array;
    }

    /**
     * Méthode pour récupérer les déplacements possibles
     * @param ech Echiquier
     * @return arrayList des déplacement possibles
     */
    public ArrayList<Case> getDepPossibles(Echiquier ech) {
        ArrayList<Case> array = new ArrayList<Case>();
        int col = this.getC();
        int lig = this.getL();

        
        for(deplacement dep : deplacement.values())
        {
            if(TestCase(ech,array, col+dep.getC(), lig+dep.getL()));
        }
        return array;

       
    }

    /**
     * Méthode pour tester les cases disponibles
     * @param ech Echiquier
     * @param array arrayList des cases
     * @param col colonnes de l'échiquier
     * @param lig lignes de l'échiquier
     * @return arrayList
     */
    private boolean TestCase(Echiquier ech, ArrayList<Case> array, int col, int lig){
        /*
        On regarde si la case existe
         */
        if(ech.caseExiste(lig,col))
        {
            /*
            On regarde si la case est vide
             */
            if(null == ech.getPiece(lig,col)){
                /*
                On l'ajoute aux cases où peut aller le roi
                 */
                array.add(new Case(lig,col));
            }
            else{
                /*
                On regarde si la case à deja une piece de la couleur du Roi
                 */
                if(ech.getPiece(lig,col).getCoul().equals(this.getCoul())){
                    return true;
                }
                else{
                    array.add(new Case(lig,col));
                   return true;
                }
            }

        }
        return false;
    }

    /**
     * Méthode pour stocker les différents déplacements possibles
     */
    public enum deplacement{

        R_HAUT(-1,0),
        R_HAUT_DROITE(-1,1),
        R_DROITE(0,1),
        R_BAS_DROITE(1,1),
        R_BAS(1,0),
        R_BAS_GAUCHE(1,-1),
        R_GAUCHE(0,-1),
        R_HAUT_GAUCHE(-1,-1);

        private int lig;
        private int col;
        
        deplacement(int lig, int col){
            this.lig = lig;
            this.col = col;
        }

        public int getL(){
            return lig;
        }

        public int getC(){
            return col;
        }
    }

    /**
     * Méthode toString qui retourne la couleur de la pièce
     * @return la couleur de la pièce
     */
    @Override
    public String toString(){
                if(this.getCoul().equals("B")){
            return "RB";
        }
        return "RN";
    }
}
